/* $Id: coords.c,v 3.2 1998/02/16 12:12:24 kas Exp $ */

/* Rutiny pro praci se souradnicemi molekuly */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h> /* for RAND_MAX */
#include <math.h>

#include "ff.h"
#include "pval.h"

int read_crd_file(pvp, filename)
potential_value *pvp;
char *filename;
{
	FILE *f;
	char buf[2560];
	int i;

	if ((f = fopen(filename, "r")) == NULL)
		return 0;
	
	/* First, we must read the name of the molecule */
	fgets (buf, 255, f);

	/* Then we must read and check # of atoms */
	fscanf(f, "%d", &i);

	if (i != pvp->ffp->natoms)
	{
		fprintf(stderr, "read_crd_file(): Wrong # of atoms\n");
		return 0;
	}

	for (i=0; i<pvp->ffp->natoms; i++)
		if (fscanf(f, "%lf %lf %lf",
			pvp->x+i, pvp->y+i, pvp->z+i) != 3)
		{
			fclose(f);
			return 0;
		}

	fclose(f);

	return 1;
}

int write_crd_file(pvp, filename, x, y, z)
potential_value *pvp;
char *filename;
REAL *x, *y, *z;
{
	FILE *f;
	int i, odd;

	if ((f = fopen(filename, "w")) == NULL)
		return 0;
	
	fputs(pvp->ffp->title, f);
	fputc('\n', f);

	fprintf(f, "%4d\n", pvp->ffp->natoms);

	for (i=0, odd=0; i<pvp->ffp->natoms; i++, odd=!odd)
	{
		fprintf(f, " % 11.7f % 11.7f % 11.7f",
			 x[i], y[i], z[i]);
		if (odd)
			fputc('\n', f);
	}
	fclose(f);
	return 1;
}

void generate_coordinates(pvp)
potential_value *pvp;
{
        int i;

        for (i=0; i<pvp->ffp->natoms; i++)
        {
                pvp->x[i] = (REAL)pvp->ffp->natoms*((REAL)rand()/RAND_MAX);
                pvp->y[i] = (REAL)pvp->ffp->natoms*((REAL)rand()/RAND_MAX);
                pvp->z[i] = (REAL)pvp->ffp->natoms*((REAL)rand()/RAND_MAX);
        }
}

REAL diff_coordinates(natoms, xyz1, xyz2)
int natoms;
REAL *xyz1, *xyz2;
{
	int i;
	REAL sumsq = 0.0;

	for (i=0; i<natoms*3; i++)
		sumsq += (xyz1[i]-xyz2[i]) * (xyz1[i]-xyz2[i]);

	return sqrt(sumsq/natoms);
}

void normalize_coordinates(natoms, xyz)
int natoms;
REAL *xyz;
{
	int i;
	REAL *x, *y, *z;
	REAL cosa1, cosa2, cosa3;
	REAL cosb1, cosb2, cosb3;
	REAL cosg1, cosg2, cosg3;

	x = xyz; y = xyz+natoms; z = xyz+2*natoms;

	/* Move the atom 0 to [0,0,0] coordinates */
	for (i=1; i<natoms; i++) {
		x[i] -= x[0];
		y[i] -= y[0];
		z[i] -= z[0];
	}
	x[0] = y[0] = z[0] = 0.0;

	/*
	 * Now we need to rotate the molecule so that the atom 1 will
	 * be on an x axis and the atom 2 on the xy plane.
	 */
	/* VYPOCITAT cos[abg][123]! */
	for (i=1; i<natoms; i++) {
		REAL x1, y1, z1;
		x1 = x[i]*cosa1 + y[i]*cosa2 + z[i]*cosa3;
		y1 = x[i]*cosb1 + y[i]*cosb2 + z[i]*cosb3;
		z1 = x[i]*cosg1 + y[i]*cosg2 + z[i]*cosg3;
		x[i] = x1;
		y[i] = y1;
		z[i] = z1;
	}
}


