/* $Id: pval.h,v 3.11 1998/02/16 12:12:26 kas Exp $ */

/* Struktura obsahujici hodnoty jednotlivych slozek siloveho pole */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef PVAL_H__
#define PVAL_H__ 1

/* Header file, obsahujici struktury pro vypocet hodnoty potencialove funkce */

#include "proto.h"

typedef struct potential_value_
{
	force_field *ffp;

	REAL value;

	REAL *nbval;	/* [ffp->natoms*ffp->natoms]
			 * M cenu tohle njak optimalizovat na pam? */
	REAL *bondval;	/* [ffp->nbonds] */
	REAL *angleval;	/* [ffp->nangles] */
	REAL *dihval;	/* [ffp->ndihs] */

	REAL *x;	/* [ffp->natoms] */
	REAL *y;	/* [ffp->natoms] */
	REAL *z;	/* [ffp->natoms] */

	REAL *dx;	/* [ffp->natoms] */
	REAL *dy;	/* [ffp->natoms] */
	REAL *dz;	/* [ffp->natoms] */

	int **nbonds;	/* [ffp->natoms][] */
	int **bonds;	/* [ffp->natoms][] */
	int **angles;	/* [ffp->natoms][] */
	int **dihs;	/* [ffp->natoms][] */

	int recomputed_atom;
	REAL ox, oy, oz;
	REAL ovalue;
	REAL *onbval;
	REAL *obondval;
	REAL *oangleval;
	REAL *odihval;
} potential_value;

#ifdef PVAL_VERBOSE
extern int pval_verbose;
#define PVAL_QUIET	0
#define PVAL_SUM	1
#define PVAL_SUMVERBOSE	2
#define PVAL_INTSUM	3
#define PVAL_DETAILED	4
#endif

potential_value *alloc_pv __P((force_field *f));
void free_pv  __P((potential_value *pvp));
int read_coordinates __P((potential_value *pvp, char *filename));
int write_coordinates __P((potential_value *pvp, char *filename));
void generate_coordinates __P((potential_value *pvp));
REAL compute_potential __P((potential_value *pvp));
REAL recompute_potential __P((potential_value *pvp, int atom, REAL nx, REAL ny, REAL nz));
REAL undo_new_potential __P((potential_value *pvp));
void compute_gradient __P((potential_value *pvp));
void compute_grad_num __P((potential_value *pvp));

int read_crd_file __P((potential_value *pvp, char *filename));
int write_crd_file __P((potential_value *pvp, char *filename, REAL *x, REAL *y, REAL *z));

REAL annealing __P((potential_value *pvp));
REAL minimize_l __P((potential_value *pvp, double accuracy));

#ifndef NONTRANSFORMED_ONLY
void setup_annealing __P((char *ofile, REAL temperature, REAL move_scale,
	REAL move_factor, int max_acc_factor, int max_factor, REAL temp_factor,
	REAL lambda0, REAL lambdaN, REAL lambda_factor,
	char *ptmpl, char *atmpl, char *ttpmp));
#else
void setup_annealing __P((char *ofile, REAL temperature, REAL move_scale,
	REAL move_factor, int max_acc_factor, int max_factor, REAL temp_factor));
#endif
#endif /* PVAL_H__ */
