/* $Id: singlept.c,v 3.4 1998/01/17 01:10:28 kas Exp $ */

/* The main() function for single-point coputing */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define PVAL_VERBOSE	1

#include "ff.h"
#include "pval.h"

static void usage();
static char *program_name;

int main(argc, argv)
int argc;
char **argv;
{
	force_field *ffp;
	potential_value *pvp;
	REAL vdw14 = -1.0, eel14 = -1.0, diel_lin = -1.0, diel_const = -1.0;
	REAL cutoff = -1.0;
	int grad = 0;
	int numeric = 0;
	int i;
#ifndef NONTRANSFORMED_ONLY
	char *pair_template = NULL, *dih_template = NULL, *angle_template = NULL;
	REAL lambda = 0.8;
#endif
	program_name = *argv;
	pval_verbose = 0; 	/* Default is not to be verbose */

        while(1)
        {
                int c, opt_ind;
                static struct option long_opts[] = {
                        { "help", 0, 0, 'h' },
                        { "version", 0, 0, 'V' },
			{ "verbose", 1, 0, 'v' },
                        { "vdw14", 1, 0, 'w' },
                        { "eel14", 1, 0, 'e' },
                        { "diel-lin", 1, 0, 'l' },
                        { "diel-const", 1, 0, 'c' },
			{ "cutoff", 1, 0, 'C' },
#ifndef NONTRANSFORMED_ONLY
			{ "pair-template", 1, 0, 'p'},
			{ "dih-template", 1, 0, 'd'},
			{ "angle-template", 1, 0, 'a'},
			{ "lambda", 1, 0, 'L'},
#endif
                        { "gradient", 0, 0, 'g' },
                        { "numgrad", 0, 0, 'n' },
                };

                if ((c=getopt_long(argc, argv, "hVgnv:w:e:l:c:C:--", long_opts,
                        &opt_ind)) == -1)
                        break;
                switch(c)
                {
                case 'h':
                case '?':
                        usage();
			/* NOTREACHED */
                case 'V':
#ifdef NONTRANSFORMED_ONLY
			printf("singlept (nontransformed force field)\n");
#else
			printf("singlept (transformed force field)\n");
#endif
                        printf("Version: $Id: singlept.c,v 3.4 1998/01/17 01:10:28 kas Exp $\n");
                        return 0;
		case 'v':
			pval_verbose = atoi(optarg);
			break;
                case 'w':
                        vdw14 = atof(optarg);
                        break;
                case 'e':
                        eel14 = atof(optarg);
                        break;
                case 'l':
                        diel_lin = atof(optarg);
                        break;
                case 'c':
                        diel_const = atof(optarg);
                        break;
		case 'C':
			cutoff = atof(optarg);
			break;
#ifndef NONTRANSFORMED_ONLY
		case 'p':
			pair_template = optarg;
			break;
		case 'a':
			angle_template = optarg;
			break;
		case 'd':
			dih_template = optarg;
			break;
		case 'L':
			lambda = atof(optarg);
			break;
#endif
		case 'g':
			grad = 1;
			break;
		case 'n':
			numeric = 1;
			break;
                default:
                	usage();
                	break;
                }
	}
	if (optind+2 > argc)
		usage();

	ffp = prm2ff(argv[optind]);

        setup_ff(ffp, vdw14, eel14, diel_lin, diel_const, cutoff);
#ifndef NONTRANSFORMED_ONLY
	read_trans_ff(ffp, lambda, angle_template, dih_template, pair_template);
#endif
	pvp = alloc_pv(ffp);

	read_crd_file(pvp, argv[optind+1]);

	if (grad || numeric) {
		REAL norm;
#if 0
		REAL d[pvp->ffp->natoms * 3];
		REAL avg, max, sum2;
		int i;
		compute_grad_num(pvp);
		for (i=0; i<pvp->ffp->natoms; i++)
			printf ("%4d: (%7.3f, %7.3f, %7.3f)\n", i,
				pvp->dx[i], pvp->dy[i], pvp->dz[i]);
		for (i=0; i<pvp->ffp->natoms * 3; i++)
			d[i] = pvp->dx[i];
		compute_gradient(pvp);
		for (i=0,avg=0,max=0,sum2=0; i<pvp->ffp->natoms; i++) {
			REAL t=fabs(d[i]-pvp->dx[i]);
			avg += t;
			if (max<t) max=t;
			sum2 += t*t;
		}
		printf("Average difference = %7.3f\n"
			"Maximum difference = %7.3f\n"
			"Sum of squares = %7.3f\n", avg/pvp->ffp->natoms,
			max, sum2);
#else
		if (numeric) {
			int old_verbose = pval_verbose;
			pval_verbose = PVAL_QUIET;
			compute_grad_num(pvp);
			pval_verbose = old_verbose;
		} else
			compute_gradient(pvp);
#endif
		for (i=0,norm=0.0; i<pvp->ffp->natoms*3; i++)
			norm += pvp->dx[i]*pvp->dx[i];
		if (pval_verbose >= PVAL_DETAILED) {
			printf("#### Atom  (    dx,        dy,        dz   )\n");
			for (i=0; i<pvp->ffp->natoms; i++)
				printf ("     %4d: (%9.3f, %9.3f, %9.3f)\n", i,
					pvp->dx[i], pvp->dy[i], pvp->dz[i]);
		}
		if (pval_verbose >= PVAL_SUMVERBOSE)
			printf("#### E2-norm of gradient:   ");
		if (pval_verbose >= PVAL_SUM)
			printf("%15.3f\n", sqrt(norm));
	} else
		compute_potential(pvp);

	return 0;
}
static void usage()
{
        fprintf(stderr, "This program computes single-point value of the"
#ifndef NONTRANSFORMED_ONLY
		" transformed"
#endif
		"\n\tpotential energy hypersurface.\n"
	 	"Usage: %s [options] file.top file.crd\n"
                "Where [options] is zero or more from\n"
                "--help\tprints out this message\n"
                "--version\tprints out the version of the program\n"
		"--verbose number\tsets the verbosity (2)\n"
                "--vdw14 number\tsets the scaling parameter for WdW 1-4 interactions (default is 2.0)\n"
                "--eel14 number\tsets the scaling parameter for E-static 1-4 interactions (1.2)\n"
#ifndef NONTRANSFORMED_ONLY
		"--lambda number\tlambda value for transfomed force field\n"
		"--pair-template string\ttemplate for pair-interactions file\n"
		"--angle-template string\ttemplate for angle-interactions file\n"
		"--dih-template string\ttemplate for torison-interactions file\n"
#endif
                "--diel-lin number\tsets the linear part of dielectric funtion (1.0)\n"
                "--diel-const number\tsets the constant part of dielectric function (0.0)\n"
		"--cutoff number\tsets the cutoff parameter for the 1991 force field (+infinity)\n"
		"--gradient\tprints the gradient values instead of potential energy value\n"
		"--numgrad\tprints the gradient values computed numerically\n",
                program_name);
        exit(1);
}

